@@ -1,7 +1,7 @@
 /*
  * gauche.h - Gauche scheme system header
  *
- *   Copyright (c) 2000-2021  Shiro Kawai  <shiro@acm.org>
+ *   Copyright (c) 2000-2023  Shiro Kawai  <shiro@acm.org>
  *
  *   Redistribution and use in source and binary forms, with or without
  *   modification, are permitted provided that the following conditions
@@ -71,16 +71,10 @@
 #include <inttypes.h>
 #include <gauche/int64.h>
 
-#ifdef TIME_WITH_SYS_TIME
+#ifdef HAVE_SYS_TIME_H
 # include <sys/time.h>
-# include <time.h>
-#else
-# ifdef HAVE_SYS_TIME_H
-#  include <sys/time.h>
-# else
-#  include <time.h>
-# endif
 #endif
+#include <time.h>
 
 typedef ssize_t ScmSize;
 
@@ -95,12 +89,12 @@ typedef ssize_t ScmSize;
 #include <gauche/extern.h>
 
 #if defined(LIBGAUCHE_BODY)
+/* We incoporate gc files within libgauche, so we need to refer to them
+   "internally" while we're compiling gauche body.  SCM_EXTERN handles it. */
+#define GC_API SCM_EXTERN
 #if !defined(GC_DLL)
 #define GC_DLL    /* for gc.h to handle Win32 crazyness */
 #endif
-#if !defined(GC_BUILD)
-#define GC_BUILD  /* ditto */
-#endif
 #endif /* LIBGAUCHE_BODY */
 #include <gc.h>
 
@@ -394,7 +388,7 @@ typedef struct ScmFlonumRec {
 #define SCM_CHAR_VALUE(obj)     SCM_CHAR(((unsigned long)SCM_WORD(obj)) >> 8)
 #define SCM_MAKE_CHAR(ch)       SCM_OBJ((intptr_t)(((unsigned long)(ch))<<8) + 3)
 
-#define SCM_CHAR_INVALID        ((ScmChar)(-1)) /* indicate invalid char */
+#define SCM_CHAR_INVALID        ((ScmChar)(-2)) /* indicate invalid char (-2 is used to avoid conflict with libc's EOF) */
 #define SCM_CHAR_MAX            (0xffffff)
 
 #define SCM_CHAR_ASCII_P(ch)    ((ch) < 0x80)
@@ -602,10 +596,6 @@ typedef ScmObj ScmSubrProc(ScmObj *, int, void*);
 #define SCM_VM(obj)          ((ScmVM *)(obj))
 #define SCM_VMP(obj)         SCM_XTYPEP(obj, SCM_CLASS_VM)
 
-#define SCM_VM_CURRENT_INPUT_PORT(vm)   (SCM_VM(vm)->curin)
-#define SCM_VM_CURRENT_OUTPUT_PORT(vm)  (SCM_VM(vm)->curout)
-#define SCM_VM_CURRENT_ERROR_PORT(vm)   (SCM_VM(vm)->curerr)
-
 SCM_EXTERN ScmVM *Scm_VM(void);     /* Returns the current VM */
 
 /* The new APIs to run Scheme code from C.
@@ -669,7 +659,12 @@ SCM_EXTERN ScmObj Scm_VMCall(ScmObj *args, int argcnt, void *data);
 SCM_EXTERN ScmObj Scm_VMCallCC(ScmObj proc);
 SCM_EXTERN ScmObj Scm_VMCallPC(ScmObj proc);
 SCM_EXTERN ScmObj Scm_VMReset(ScmObj proc);
+SCM_EXTERN void   Scm_VMPushDynamicHandlers(ScmObj, ScmObj, ScmObj);
 SCM_EXTERN ScmObj Scm_VMDynamicWind(ScmObj pre, ScmObj body, ScmObj post);
+SCM_EXTERN int    Scm_ContinuationP(ScmObj proc);
+
+SCM_EXTERN ScmObj Scm_VMCallWithContinuationPrompt(ScmObj, ScmObj, ScmObj);
+SCM_EXTERN ScmObj Scm_VMAbortCurrentContinuation(ScmObj, ScmObj);
 
 SCM_EXTERN ScmObj Scm_VMWithErrorHandler(ScmObj handler, ScmObj thunk);
 SCM_EXTERN ScmObj Scm_VMWithGuardHandler(ScmObj handler, ScmObj thunk);
@@ -877,7 +872,7 @@ enum {
 
 SCM_EXTERN ScmObj Scm_Cons(ScmObj car, ScmObj cdr);
 SCM_EXTERN ScmObj Scm_Acons(ScmObj caar, ScmObj cdar, ScmObj cdr);
-SCM_EXTERN ScmObj Scm_MakeImmutablePair(ScmObj car, ScmObj cdr);
+SCM_EXTERN ScmObj Scm_MakeImmutablePair(ScmObj car, ScmObj cdr, ScmObj attrs);
 SCM_EXTERN ScmObj Scm_List(ScmObj elt, ...);
 SCM_EXTERN ScmObj Scm_Conses(ScmObj elt, ...);
 SCM_EXTERN ScmObj Scm_VaList(va_list elts);
@@ -902,6 +897,7 @@ SCM_EXTERN void   Scm_SetCdr(ScmObj pair, ScmObj value);
 SCM_EXTERN ScmSize Scm_Length(ScmObj obj);
 SCM_EXTERN ScmObj Scm_CopyList(ScmObj list);
 SCM_EXTERN ScmObj Scm_MakeList(ScmSmallInt len, ScmObj fill);
+SCM_EXTERN ScmObj Scm_AlistCopy(ScmObj alist);
 SCM_EXTERN ScmObj Scm_Append2X(ScmObj list, ScmObj obj);
 SCM_EXTERN ScmObj Scm_Append2(ScmObj list, ScmObj obj);
 SCM_EXTERN ScmObj Scm_Append(ScmObj args);
@@ -1221,7 +1217,7 @@ struct ScmProcedureRec {
       is compiled as the ordinary procedure call.
 
    <procedure>: A procedural inliner.  It has signature Sexpr,[IForm] -> IForm,
-      where Sexpr is the original source of call size (just for debug info) and
+      where Sexpr is the original source of call site (just for debug info) and
       input [IForm] is the IForm for list of arguments.  See compiler-1.scm.
       It returns the modified IForm.  It can return #<undef>, to indicate
       inlining isn't possible.
@@ -1500,7 +1496,6 @@ SCM_CLASS_DECL(Scm_PromiseClass);
 #define SCM_PROMISE(obj)            ((ScmPromise*)(obj))
 #define SCM_PROMISEP(obj)           SCM_XTYPEP(obj, SCM_CLASS_PROMISE)
 
-SCM_EXTERN ScmObj Scm_MakePromise(int forced, ScmObj code);
 SCM_EXTERN ScmObj Scm_VMForce(ScmObj p); /* CPS, lightweight */
 SCM_EXTERN ScmObj Scm_Force(ScmObj p);
 
@@ -1690,6 +1685,12 @@ SCM_EXTERN void   Scm_SetSigmask(sigset_t *mask);
 
 #include <gauche/load.h>
 
+/*---------------------------------------------------
+ * THREADS
+ */
+
+#include <gauche/thread.h>
+
 /*---------------------------------------------------
  * PROFILER INTERFACE
  */
