@@ -1,7 +1,7 @@
 /*
  * gauche/bits.h - Bit manipulation utilities
  *
- *   Copyright (c) 2007-2022  Shiro Kawai  <shiro@acm.org>
+ *   Copyright (c) 2007-2023  Shiro Kawai  <shiro@acm.org>
  *
  *   Redistribution and use in source and binary forms, with or without
  *   modification, are permitted provided that the following conditions
@@ -120,4 +120,7 @@ SCM_EXTERN int    Scm_BitsLowest0(const ScmBits *bits, int start, int end);
 SCM_EXTERN int    Scm_BitsHighest1(const ScmBits *bits, int start, int end);
 SCM_EXTERN int    Scm_BitsHighest0(const ScmBits *bits, int start, int end);
 
+SCM_EXTERN int    Scm_BitsAny(const ScmBits *bits, int start, int end);
+SCM_EXTERN int    Scm_BitsEvery(const ScmBits *bits, int start, int end);
+
 #endif /*GAUCHE_BITS_H*/
