@@ -1,7 +1,7 @@
 /*
  * number.h - Public API for Scheme numbers
  *
- *   Copyright (c) 2000-2022  Shiro Kawai  <shiro@acm.org>
+ *   Copyright (c) 2000-2023  Shiro Kawai  <shiro@acm.org>
  *
  *   Redistribution and use in source and binary forms, with or without
  *   modification, are permitted provided that the following conditions
@@ -260,6 +260,9 @@ SCM_EXTERN ScmObj    Scm_OffsetToInteger(off_t);
 SCM_EXTERN int       Scm_IntegerFitsIntptrP(ScmObj);
 SCM_EXTERN intptr_t  Scm_IntegerToIntptr(ScmObj);
 SCM_EXTERN ScmObj    Scm_IntptrToInteger(intptr_t);
+SCM_EXTERN int       Scm_IntegerFitsUintptrP(ScmObj);
+SCM_EXTERN uintptr_t Scm_IntegerToUintptr(ScmObj);
+SCM_EXTERN ScmObj    Scm_UintptrToInteger(uintptr_t);
 
 SCM_EXTERN ScmObj Scm_MakeRational(ScmObj numer, ScmObj denom);
 SCM_EXTERN ScmObj Scm_MakeRatnum(ScmObj numer, ScmObj denom);
@@ -334,6 +337,7 @@ SCM_EXTERN ScmObj Scm_LogIor(ScmObj x, ScmObj y);
 SCM_EXTERN ScmObj Scm_LogXor(ScmObj x, ScmObj y);
 SCM_EXTERN ScmObj Scm_LogNot(ScmObj x);
 SCM_EXTERN ScmObj Scm_Ash(ScmObj x, ScmSmallInt cnt);
+SCM_EXTERN u_long Scm_IntegerLength(ScmObj n);
 
 enum ScmRoundMode {
     SCM_ROUND_FLOOR,
