@@ -1,7 +1,7 @@
 /*
  * gauche/port.h - Port API
  *
- *   Copyright (c) 2000-2022  Shiro Kawai  <shiro@acm.org>
+ *   Copyright (c) 2000-2023  Shiro Kawai  <shiro@acm.org>
  *
  *   Redistribution and use in source and binary forms, with or without
  *   modification, are permitted provided that the following conditions
@@ -87,7 +87,8 @@ typedef struct ScmPortBufferRec {
     int  (*ready)(ScmPort *p);
     int  (*filenum)(ScmPort *p);
     off_t (*seeker)(ScmPort *p, off_t offset, int whence);
-    void *data;
+    void *data;         /* aux data to be used in handlers (heap allocated) */
+    long idata;         /* aux data to be used in handlers (single word) */
 
     /* The following fields are added in 0.9.10.  When you pass ScmPortBuffer
        to the constructor, those fields are looked at only when you specify
@@ -111,7 +112,8 @@ typedef struct ScmPortVTableRec {
     void    (*Flush)(ScmPort *p);
     void    (*Close)(ScmPort *p);
     off_t   (*Seek)(ScmPort *p, off_t off, int whence);
-    void    *data;
+    void    *data;      /* aux data to be used in handlers (heap allocated) */
+    long    idata;      /* aux data to be used in handlers (single word) */
 
     /* The following fields are added in 0.9.10.  When you pass ScmPortBuffer
        to the constructor, those fields are looked at only when you specify
@@ -205,7 +207,7 @@ struct ScmPortRec {
        We tested to use a pointer to the real struct, but I/O intensive
        benchmark showed 2-3% reduction of speed.  So we avoid indirection.
      */
-    char opaque[29*sizeof(ScmWord) + 2*sizeof(ScmInternalFastlock)];
+    char opaque[30*sizeof(ScmWord) + 2*sizeof(ScmInternalFastlock)];
 };
 
 /* Port direction.
@@ -222,6 +224,14 @@ enum ScmPortDirection {
     SCM_PORT_OUTPUT_TRANSIENT = 4+SCM_PORT_OUTPUT
 };
 
+/*
+ * The 'mode' flags
+ *
+ *   Bit 0, 1, 2 - ScmPortBufferingMode
+ *   Bit 3-7 - Reserved
+ *   Bit 8 - SCM_PORT_BUFFER_SIGPIPE_SENSITIVE
+ */
+
 /* Port buffering mode */
 enum ScmPortBufferMode {
     SCM_PORT_BUFFER_FULL = 0,       /* full buffering */
@@ -347,9 +357,13 @@ SCM_EXTERN int    Scm_PortPositionable(ScmPort *port, int setp);
 
 SCM_EXTERN void   Scm_FlushAllPorts(int exitting);
 
+SCM_EXTERN void   Scm_LinkPorts(ScmPort *iport, ScmPort *oport);
+SCM_EXTERN void   Scm_UnlinkPorts(ScmPort *port);
+
 SCM_EXTERN ScmObj Scm_PortName(ScmPort *port);
 SCM_EXTERN ScmSize Scm_PortLine(ScmPort *port);
 SCM_EXTERN ScmSize Scm_PortBytes(ScmPort *port);
+SCM_EXTERN ScmSize Scm_PortColumn(ScmPort *port);
 SCM_EXTERN ScmObj Scm_GetPortPosition(ScmPort *port);
 SCM_EXTERN ScmObj Scm_GetPortPositionUnsafe(ScmPort *port);
 SCM_EXTERN ScmObj Scm_SetPortPosition(ScmPort *port, ScmObj pos);
@@ -435,9 +449,13 @@ SCM_EXTERN ScmObj Scm_SetStdin(ScmPort *port);
 SCM_EXTERN ScmObj Scm_SetStdout(ScmPort *port);
 SCM_EXTERN ScmObj Scm_SetStderr(ScmPort *port);
 
-#define SCM_CURIN    SCM_VM_CURRENT_INPUT_PORT(Scm_VM())
-#define SCM_CUROUT   SCM_VM_CURRENT_OUTPUT_PORT(Scm_VM())
-#define SCM_CURERR   SCM_VM_CURRENT_ERROR_PORT(Scm_VM())
+SCM_EXTERN ScmPort *Scm_CurrentInputPort(void);
+SCM_EXTERN ScmPort *Scm_CurrentOutputPort(void);
+SCM_EXTERN ScmPort *Scm_CurrentErrorPort(void);
+
+#define SCM_CURIN    Scm_CurrentInputPort()
+#define SCM_CUROUT   Scm_CurrentOutputPort()
+#define SCM_CURERR   Scm_CurrentErrorPort()
 
 SCM_EXTERN ScmObj Scm_SetCurrentInputPort(ScmPort *port);
 SCM_EXTERN ScmObj Scm_SetCurrentOutputPort(ScmPort *port);
