@@ -1,7 +1,7 @@
 /*
  * string.h - Public API for Scheme strings
  *
- *   Copyright (c) 2000-2021  Shiro Kawai  <shiro@acm.org>
+ *   Copyright (c) 2000-2023  Shiro Kawai  <shiro@acm.org>
  *
  *   Redistribution and use in source and binary forms, with or without
  *   modification, are permitted provided that the following conditions
@@ -240,6 +240,9 @@ SCM_EXTERN char **Scm_ListToCStringArray(ScmObj lis, int errp,
 SCM_EXTERN ScmObj  Scm_StringToList(ScmString *str);
 SCM_EXTERN ScmObj  Scm_ListToString(ScmObj chars);
 
+SCM_EXTERN ScmObj  Scm_ReadStringLiteral(ScmPort*, ScmReadContext*,
+                                         u_long, ScmChar);
+
 /* OBSOLETED */
 SCM_EXTERN ScmObj  Scm_StringCompleteToIncomplete(ScmString *);
 SCM_EXTERN ScmObj  Scm_StringIncompleteToComplete(ScmString *, int, ScmChar);
