@@ -1,7 +1,7 @@
 /*
  * vm.h - Virtual machine
  *
- *   Copyright (c) 2000-2022  Shiro Kawai  <shiro@acm.org>
+ *   Copyright (c) 2000-2023  Shiro Kawai  <shiro@acm.org>
  *
  *   Redistribution and use in source and binary forms, with or without
  *   modification, are permitted provided that the following conditions
@@ -74,11 +74,13 @@ typedef struct ScmCompiledCodeRec ScmCompiledCode;
 /* Actual structure is defined in priv/vmP.h */
 typedef struct ScmCallTraceRec ScmCallTrace;
 
-/* Actual structure is defined in priv/parameterP.h */
-typedef struct ScmVMParameterTableRec ScmVMParameterTable;
+/* Actual structure is defined in priv/vmP.h */
+typedef struct ScmPromptTagRec ScmPromptTag;
+typedef struct ScmThreadLocalRec ScmThreadLocal;
+typedef struct ScmVMThreadLocalTableRec ScmVMThreadLocalTable;
 
 /*
- * Environment frame
+ * Static environment frame
  *
  *   :        :
  *   +--------+
@@ -104,6 +106,31 @@ typedef struct ScmEnvFrameRec {
 #define ENV_FP(env)        (((ScmObj*)(env))-((env)->size))
 #define ENV_DATA(env, num) (*(((ScmObj*)(env))-(num)-1))
 
+/*
+ * Dynamic environment
+ *
+ *  Dynamic environment is just an alist pointed from vm->denv.
+ *  It is a part of continuation---when a continuation frame is pushed,
+ *  the tip of denv is saved with it.  When a continuation frame is
+ *  popped, the saved denv is restored.
+ *  This means that you don't ever need to worry about popping denv.
+ */
+
+SCM_EXTERN void   Scm_VMPushDynamicEnv(ScmObj key, ScmObj val);
+SCM_EXTERN ScmObj Scm_VMFindDynamicEnv(ScmObj key, ScmObj fallback);
+
+typedef struct ScmContinuationMarkSetRec ScmContinuationMarkSet;
+
+SCM_CLASS_DECL(Scm_ContinuationMarkSetClass);
+#define SCM_CLASS_CONTINUATION_MARK_SET (&Scm_ContinuationMarkSetClass)
+#define SCM_CONTINUATION_MARK_SET(obj)  ((ScmContinuationMarkSet*)obj)
+#define SCM_CONTINUATION_MARK_SET_P(obj) SCM_ISA(obj,SCM_CLASS_CONTINUATION_MARK_SET)
+
+SCM_EXTERN ScmObj Scm_ContinuationMarks(ScmObj contProc, ScmObj promptTag);
+SCM_EXTERN ScmObj Scm_CurrentContinuationMarks(ScmObj promptTag);
+SCM_EXTERN ScmObj Scm_ContinuationMarkSetToList(const ScmContinuationMarkSet *,
+                                                ScmObj, ScmObj);
+
 /*
  * Continuation frame
  *
@@ -116,6 +143,7 @@ typedef struct ScmEnvFrameRec {
  *   |  cpc   |
  *   | marker |
  *   | size=N |
+ *   |  denv  |
  *   |  env   |
  *   |..prev..|<--- ScmContFrame* cont
  *   |arg[N-1]|
@@ -125,8 +153,39 @@ typedef struct ScmEnvFrameRec {
  *   +--------+
  *   :        :
  *
- *  If env is a special value (&ccEnvMark), this is a C continuation
- *  and pc contains a C function pointer.
+ *  If env is a special value (&ccEnvMark), this is a C continuation.
+ *  Some fields are used differently:
+ *
+ *   |  base   |
+ *   |   pc    |  <-- PCont procedure
+ *   |  cpc    |  <-- CCont procedure, NULL, or #<dynamic-handler>
+ *   | marker  |
+ *   | size=N  |
+ *   |  denv   |
+ *   |  env    |  <-- &ccEnvMark
+ *   |..prev.. |
+ *   |data[N-1]|
+ *   |data[N-2]|
+ *   :         :
+ *   | data[0] |
+ *   +---------+
+ *
+ *  The difference between CCont and PCont is purely historical.
+ *  CCont is the original C continuation.  It takes one result and
+ *  the data array.  PCont is introduced to support precompile-to-C
+ *  code, and it takes ScmVM* in addition to the result and data array.
+ *
+ *  When C continuation frame is popped, the value of cpc is temporaily
+ *  saved in vm->ccont, and PCont procedure is invoked with vm, val0,
+ *  and pointer to the data array.  If the C continuation frame is
+ *  pushed with Scm_pc_PushCC, vm->ccont is NULL (unused).
+ *  If the C continuation frame is pushed with Scm_VMPushCC, PCont
+ *  procedure is set to ccont_adapter() which invokes vm->ccont as
+ *  CCont procedure.
+ *
+ *  A special case is a C continuation frame pushed for execution of
+ *  body thunk of dynamic-wind.  It contains #<dynamic-handler>,
+ *  so that stack rewinder can find dynamic handlers.
  */
 
 /* NB: The size of continuation frame (in words) is embedded in the precompiled
@@ -141,9 +200,10 @@ typedef struct ScmEnvFrameRec {
 typedef struct ScmContFrameRec {
     struct ScmContFrameRec *prev; /* previous frame */
     ScmEnvFrame *env;             /* saved environment */
+    ScmObj denv;                  /* dynamic environment links */
 #if SIZEOF_LONG == 4
-    long size : 31;               /* size of argument frame */
-    u_long marker : 1;            /* end marker of partial continuation */
+    long size : 30;               /* size of argument frame */
+    u_long marker : 2;            /* end marker of partial continuation */
 #else
     int size;                     /* size of argument frame */
     int marker;                   /* end marker of partial continuation */
@@ -161,28 +221,45 @@ SCM_EXTERN int  Scm__VMProtectStack(ScmVM *vm);
 SCM_EXTERN void Scm__VMUnprotectStack(ScmVM *vm);
 
 /*
- * Syntactic closure
- *
- *   Syntactic closure encapsulates compile-time environment for
- *   hygienic macro expansion.
- *   See Bawden & Rees, Syntactic Closures, MIT AI Memo 1049, June 1988.
+ * Thread Locals
+ *  See also priv/vmP.h for private definitions
  */
 
-typedef struct ScmSyntacticClosureRec {
-    ScmObj env;                 /* compile-time environment */
-    ScmObj literals;            /* literal symbols */
-    ScmObj expr;                /* expression */
-} ScmSyntacticClosure;
+SCM_CLASS_DECL(Scm_ThreadLocalClass);
+#define SCM_CLASS_THREAD_LOCAL  (&Scm_ThreadLocalClass)
+#define SCM_THREAD_LOCAL(obj)   ((ScmThreadLocal*)obj)
+#define SCM_THREAD_LOCAL_P(obj) SCM_ISA(obj,SCM_CLASS_THREAD_LOCAL)
 
-SCM_CLASS_DECL(Scm_SyntacticClosureClass);
-#define SCM_CLASS_SYNTACTIC_CLOSURE   (&Scm_SyntacticClosureClass)
+/* Flag value for Scm_MakeThreadLocal */
+enum {
+    /* Whether the initial value is inherited from the parent thread.
+       Thread locals are noninheritable by default, while parameters are
+       inheritable. */
+    SCM_THREAD_LOCAL_INHERITABLE = (1UL << 0)
 
-#define SCM_SYNTACTIC_CLOSURE(obj)   ((ScmSyntacticClosure*)(obj))
-#define SCM_SYNTACTIC_CLOSURE_P(obj) SCM_XTYPEP(obj, SCM_CLASS_SYNTACTIC_CLOSURE)
+};
 
-SCM_EXTERN ScmObj Scm_MakeSyntacticClosure(ScmObj env,
-                                           ScmObj literals,
-                                           ScmObj expr);
+SCM_EXTERN ScmThreadLocal *Scm_MakeThreadLocal(ScmObj name,
+                                               ScmObj initval,
+                                               u_long flags);
+SCM_EXTERN ScmObj Scm_ThreadLocalRef(ScmVM *vm,
+                                     const ScmThreadLocal *tl);
+SCM_EXTERN ScmObj Scm_ThreadLocalSet(ScmVM *vm,
+                                     const ScmThreadLocal *tl,
+                                     ScmObj val);
+
+
+/*
+ * Prompt tags
+ */
+
+SCM_CLASS_DECL(Scm_PromptTagClass);
+#define SCM_CLASS_PROMPT_TAG   (&Scm_PromptTagClass)
+#define SCM_PROMPT_TAG(obj)    ((ScmPromptTag*)obj)
+#define SCM_PROMPT_TAG_P(obj)  SCM_XTYPEP(obj, SCM_CLASS_PROMPT_TAG)
+
+SCM_EXTERN ScmObj Scm_MakePromptTag(ScmObj name);
+SCM_EXTERN ScmObj Scm_DefaultPromptTag();
 
 /*
  * Identifier
@@ -402,26 +479,33 @@ struct ScmVMRec {
                                    ext/threads/threads.c, and turned off by
                                    process_queued_requests() in vm.c */
 
-    ScmPort *curin;             /* current input port */
-    ScmPort *curout;            /* current output port */
-    ScmPort *curerr;            /* current error port */
-    ScmVMParameterTable *parameters; /* parameter table */
+    ScmVMThreadLocalTable *threadLocals; /* thread local table */
 
     /* Registers */
     ScmCompiledCode *base;      /* Current executing closure's code packet. */
     SCM_PCTYPE pc;              /* Program pointer.  Points into the code
                                    vector. (base->code) */
     ScmEnvFrame *env;           /* Current environment.                      */
+    ScmObj denv;                /* Current dynamic environment. */
     ScmContFrame *cont;         /* Current continuation.                     */
     ScmObj *argp;               /* Current argument pointer.  Points
                                    to the incomplete environment frame
                                    being accumulated.  This is a part of
                                    continuation.                             */
+    void *ccont;                /* This is to support old C continuation
+                                   protocol with the new protool.  It is
+                                   set with CCont procedure right before
+                                   PCont procedure is called.  See the
+                                   comment on Scm_VMPushCC in vm.c.
+                                */
     ScmObj val0;                /* Value register.                           */
     ScmObj vals[SCM_VM_MAX_VALUES]; /* Value register for multiple values */
     int    numVals;             /* # of values */
+    int    trampoline;          /* Usually -1. C-compiled subr may set >=0
+                                   before returning to trampoline to another
+                                   subr. */
 
-    ScmObj handlers;            /* chain of active dynamic handlers          */
+    ScmObj dynamicHandlers;     /* chain of active dynamic handlers          */
 
     ScmObj *sp;                 /* stack pointer */
     ScmObj *stack;              /* bottom of allocated stack area */
@@ -445,20 +529,18 @@ struct ScmVMRec {
 #endif
 
     /* Escape handling */
-    ScmObj exceptionHandler;    /* the current exception handler installed by
-                                   with-exception-handler. */
-    ScmObj exceptionHandlerStack; /* chained previous handlers. */
     ScmEscapePoint *escapePoint;/* chain of escape points (a kind of one-shot
                                    continuation).  used by system's default
                                    exception handler to escape from the error
                                    handlers. */
-    ScmEscapePoint *escapePointFloating;
-                                /* reverse link of escape point chain
-                                   to keep 'active' EPs.
-                                   See ScmEscapePoint definition above. */
     int escapeReason;           /* temporary storage to pass data across
                                    longjmp(). */
     void *escapeData[2];        /* ditto. */
+    int errorHandlerContinuable; /* A transient flag, set by the error handler
+                                    as a result of 'guard' expansion to tell
+                                    that the control should return to 'raise'
+                                    after handler returns.  See %reraise
+                                    in libexc.scm as well. */
 
     /* Custom debugger or error reporter */
     ScmObj customErrorReporter; /* If set, Scm_ReportError (report-error) calls
@@ -508,10 +590,16 @@ SCM_EXTERN int    Scm_AttachVM(ScmVM *vm);
 SCM_EXTERN void   Scm_DetachVM(ScmVM *vm);
 SCM_EXTERN void   Scm_VMDump(ScmVM *vm);
 SCM_EXTERN ScmObj Scm_VMDefaultExceptionHandler(ScmObj exc);
+SCM_EXTERN ScmObj Scm_VMCurrentExceptionHandler();
+SCM_EXTERN ScmObj Scm_VMExceptionHandlerStack();
+SCM_EXTERN void   Scm_VMPushExceptionHandler(ScmObj eh);
+SCM_EXTERN ScmObj Scm_VMPopExceptionHandler();
+SCM_EXTERN ScmObj Scm_VMWithExceptionHandler(ScmObj eh, ScmObj thunk);
 SCM_EXTERN ScmObj Scm_VMThrowException(ScmVM *vm, ScmObj exc, u_long flags);
 SCM_EXTERN ScmObj Scm_VMGetSourceInfo(ScmCompiledCode *code, SCM_PCTYPE pc);
 SCM_EXTERN ScmObj Scm_VMGetBindInfo(ScmCompiledCode *code, SCM_PCTYPE pc);
 SCM_EXTERN void   Scm_VMSetResult(ScmObj obj);
+SCM_EXTERN ScmVM *Scm_VMTakeSnapshot(ScmVM *master);
 
 SCM_CLASS_DECL(Scm_VMClass);
 #define SCM_CLASS_VM              (&Scm_VMClass)
@@ -613,6 +701,10 @@ SCM_EXTERN long Scm_VMUnwindProtect(ScmVM *vm, ScmCStack *cstack);
 SCM_EXTERN void Scm_VMNextHandler(ScmVM *vm);
 SCM_EXTERN void Scm_VMRewindProtect(ScmVM *vm);
 
+SCM_EXTERN ScmObj Scm_VMGetDynamicHandlers(void);
+SCM_EXTERN void   Scm_VMSetDynamicHandlers(ScmObj handlers);
+SCM_EXTERN void   Scm_VMFlushDynamicHandlers(void);
+
 #if GAUCHE_API_VERSION < 98
 /* Deprecated - not that useful */
 SCM_EXTERN ScmObj Scm_VMDynamicWindC(ScmSubrProc *before,
@@ -656,8 +748,6 @@ SCM_EXTERN void Scm_VMPushCC(ScmCContinuationProc *func,
                              void **data,
                              int datasize);
 
-#define SCM_CCONT_DATA_SIZE 6   /* Maximum datasize for VMPushCC */
-
 /*
  * Compiler flags
  */
