@@ -1,7 +1,7 @@
 ;;;
 ;;; compat.jfilter - jfilter compatibility interface
 ;;;
-;;;   Copyright (c) 2000-2022  Shiro Kawai  <shiro@acm.org>
+;;;   Copyright (c) 2000-2023  Shiro Kawai  <shiro@acm.org>
 ;;;
 ;;;   Redistribution and use in source and binary forms, with or without
 ;;;   modification, are permitted provided that the following conditions
@@ -44,7 +44,7 @@
 
 (define-module compat.jfilter
   (use gauche.charconv)
-  (use srfi-13)
+  (use srfi.13)
   (export cv-file
           cv-string
           judge-file))
