@@ -1,7 +1,7 @@
 ;;;
 ;;; data.imap - immutable tree map
 ;;;
-;;;   Copyright (c) 2015-2019  Shiro Kawai  <shiro@acm.org>
+;;;   Copyright (c) 2015-2023  Shiro Kawai  <shiro@acm.org>
 ;;;
 ;;;   Redistribution and use in source and binary forms, with or without
 ;;;   modification, are permitted provided that the following conditions
@@ -40,7 +40,7 @@
   (use gauche.record)
   (use data.queue)
   (use util.match)
-  (use srfi-114)
+  (use srfi.114)
   (export <imap> <imap-meta>
           make-imap alist->imap tree-map->imap
           imap? imap-empty?
@@ -144,7 +144,7 @@
      (unless (comparator? cmpr)
        (error "comparator required, but got:" cmpr))
      (make <imap> :comparator cmpr)]
-    [(key=? key<?) (make-imap (%key-proc->comparator key=? key<?))])) 
+    [(key=? key<?) (make-imap (%key-proc->comparator key=? key<?))]))
 
 ;; API
 (define alist->imap
@@ -247,6 +247,10 @@
   (proc (^p (push! alist p))
         (^[] (alist->imap alist comparator))))
 
+;; Key comparator
+(define (imap-comparator imap)
+  (slot-ref imap 'comparator))
+
 ;; A couple of conversion methods for the efficiency
 (define-method coerce-to ((c <imap-meta>) (src <list>))
   (alist->imap src))
