@@ -1,7 +1,7 @@
 ;;;
 ;;; data.range - Range objects
 ;;;
-;;;   Copyright (c) 2021-2022  Shiro Kawai  <shiro@acm.org>
+;;;   Copyright (c) 2021-2023  Shiro Kawai  <shiro@acm.org>
 ;;;
 ;;;   Redistribution and use in source and binary forms, with or without
 ;;;   modification, are permitted provided that the following conditions
@@ -31,20 +31,21 @@
 ;;;   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ;;;
 
-;; Superset of srfi-196
+;; Superset of SRFI-196
 
 (define-module data.range
   (use gauche.sequence)
   (use scheme.list)
   (use scheme.division)
-  (use srfi-42)
+  (use srfi.42)
   (export
    ;; we expose only the root classes (not the subclasses; they're
    ;; internal details)
    <range-meta> <range>
 
    ;; Constructors
-   range numeric-range iota-range vector-range uvector-range string-range
+   range numeric-range iota-range vector-range string-range
+   uvector-range bitvector/bool-range bitvector/int-range
    range-append range-reverse
 
    ;; Predicates
@@ -74,7 +75,7 @@
 (select-module data.range)
 
 ;; The classes <range> and <flat-range> are private.  Users must use
-;; srfi-196 API.
+;; SRFI-196 API.
 
 ;; In Gauche, a range is a sequence whose element may be computed from
 ;; the index.
@@ -89,20 +90,32 @@
   (format port "#<range (~d)>" (~ r'length)))
 
 ;; special case, where indexer is simply a indexed reference from a sequence
+;; raw-indexer, storage and offset must be given at initialization; indexer
+;; is computed based on the initialized value.  Changing them later will have
+;; no effect.
 (define-class <flat-range> (<range>)
-  ((indexer :init-value (^[o i]
-                          (assume (exact-integer? i))
-                          (assume (< -1 i (~ o'length)))
-                          (let1 off (+ (~ o'offset) i)
-                            (ref (~ o'storage) off))))
+  ((raw-indexer :init-keyword :raw-indexer :init-value ref)
    (storage :init-keyword :storage)
    (offset :init-keyword :offset :init-value 0)))
 
+(define-method initialize ((r <flat-range>) initargs)
+  (next-method)
+  (let ([raw-indexer (~ r'raw-indexer)]
+        [offset (~ r'offset)]
+        [storage (~ r'storage)]
+        [length (~ r'length)])
+    (set! (~ r'indexer)
+          (^[o i]
+            (assume (and (exact-integer? i)
+                         (<=:< 0 i length))
+                    "Index out of range:" i)
+            (raw-indexer storage (+ offset i))))))
+
 ;; appended ranges.
 ;; ranges slot contains #((offset . subrange) ...)
 (define (%appended-range-index range i)
-  (assume (exact-integer? i))
-  (assume (< -1 i (~ range'length)))
+  (assume (exact-integer? i) "Range index must be an exact integer:" i)
+  (assume (<=:< 0 i (~ range'length)) "Range index out of range:" i)
   (let* ([vec (~ range'ranges)]
          [len (vector-length vec)])
     (define (call-indexer p i)
@@ -141,7 +154,8 @@
 ;;;
 
 (define (range length indexer)
-  (assume (and (exact-integer? length) (>= length 0)))
+  (assume (and (exact-integer? length) (>= length 0))
+          "Range length must be exact nonnnegative integer:" length)
   (make <range> :length length :indexer (^[_ i] (indexer i))))
 
 (define (numeric-range start end :optional (step 1))
@@ -157,17 +171,19 @@
 ;; optional arguments are Gauche-specific
 (define (vector-range vec :optional (start 0) (end (vector-length vec)))
   (assume-type vec <vector>)
-  (assume (<= 0 start end (vector-length vec)))
+  (assume (<= 0 start end (vector-length vec))
+          "Start or end index is inconsistent:" start end)
   (make <flat-range>
     :length (- end start)
-    :indexer vector-ref
+    :raw-indexer vector-ref
     :storage vec
     :offset start))
 
 ;; optional arguments are Gauche-specific
 (define (string-range str :optional (start 0) (end (string-length str)))
   (assume-type str <string>)
-  (assume (<= 0 start end (string-length str)))
+  (assume (<= 0 start end (string-length str))
+          "Start or end index is inconsistent:" start end)
   (make <flat-range>
     :length (- end start)
     :indexer string-ref
@@ -177,12 +193,38 @@
 ;; Gauche extension
 (define (uvector-range uvec :optional (start 0) (end (uvector-length uvec)))
   (assume-type uvec <uvector>)
-  (assume (<= 0 start end (uvector-length uvec)))
+  (assume (<= 0 start end (uvector-length uvec))
+          "Start or end index is inconsistent:" start end)
   (make <flat-range>
     :length (- end start)
+    :raw-indexer uvector-ref
     :storage uvec
     :offset start))
 
+;; Gauche extension
+(define (bitvector/bool-range bvec
+                              :optional (start 0) (end (bitvector-length bvec)))
+  (assume-type bvec <bitvector>)
+  (assume (<= 0 start end (bitvector-length bvec))
+          "Start or end index is inconsistent:" start end)
+  (make <flat-range>
+    :length (- end start)
+    :raw-indexer bitvector-ref/bool
+    :storage bvec
+    :offset start))
+
+;; Gauche extension
+(define (bitvector/int-range bvec
+                              :optional (start 0) (end (bitvector-length bvec)))
+  (assume-type bvec <bitvector>)
+  (assume (<= 0 start end (bitvector-length bvec))
+          "Start or end index is inconsistent:" start end)
+  (make <flat-range>
+    :length (- end start)
+    :raw-indexer bitvector-ref/int
+    :storage bvec
+    :offset start))
+
 (define (range-append . ranges)
   (receive (off&ranges total)
       (map-accum (^[range total]
@@ -195,7 +237,8 @@
 
  (define (range-reverse rg :optional (start 0) (end (range-length rg)))
   (assume-type rg <range>)
-  (assume (<= 0 start end (range-length rg)))
+  (assume (<= 0 start end (range-length rg))
+          "Start or end index is inconsistent:" start end)
   (if (is-a? rg <subrange>)
     (if (%subrange-reversed? rg)
       (make <subrange>
@@ -221,7 +264,8 @@
 
 (define (range=? elt= . ranges)
   (or (null? ranges)
-      (and (assume (range? (car ranges))) (null? (cdr ranges)))
+      (and (assume (range? (car ranges)) "Range required, but got:" (car ranges))
+           (null? (cdr ranges)))
       (let1 len (range-length (car ranges))
         (every?-ec (: r (cdr ranges))
                    (and (= len (range-length r))
@@ -234,7 +278,7 @@
 ;;;
 
 (define (range-length range)
-  (assume (range? range))
+  (assume (range? range) "Range required, but got:" range)
   (~ range'length))
 
 ;; The optional fallback is Gauche extension
@@ -270,9 +314,9 @@
 
 (define (subrange range start end)
   (assume-type range <range>)
-  (if (> (- end start) (range-length range))
-    (error "subrange start/end index out of original range:"
-           (list range start end)))
+  (assume (<= start end (range-length range))
+          "Subrange start/end index out of original range:"
+          range start end)
   (if (is-a? range <subrange>)
     ;; We flatten the range
     (make <subrange>
@@ -561,6 +605,6 @@
 
 ;; some shortcuts
 (define-method subseq ((r <range>) s e) (subrange r s e))
-(define-method subseq ((r <range>) s) (subrange r s (range-length e)))
+(define-method subseq ((r <range>) s) (subrange r s (range-length r)))
 (define-method coerce-to ((r <range-meta>) (v <vector>)) (vector-range v))
 (define-method coerce-to ((r <range-meta>) (v <string>)) (string-range v))
