@@ -1,7 +1,7 @@
 ;;;
 ;;; gauche.cgen.bbb - Basic-blocks backend
 ;;;
-;;;   Copyright (c) 2021-2022  Shiro Kawai  <shiro@acm.org>
+;;;   Copyright (c) 2021-2023  Shiro Kawai  <shiro@acm.org>
 ;;;
 ;;;   Redistribution and use in source and binary forms, with or without
 ;;;   modification, are permitted provided that the following conditions
@@ -36,13 +36,19 @@
   ;; we want to have a separate module that exposes some compiler internals.
   (extend gauche.internal)
   (use gauche.sequence)
+  (use scheme.list)
   (use scheme.set)
   (use util.match)
   (use gauche.vm.insn)
   (export compile-b
           compile-b/dump  ; for debugging
+
+          bb-name bb-incoming-regs
+
+          cluster-regs
+          cluster-needs-dispatch?
+
           dump-benv
-          bb-name
           <reg>
           <const> const-value)
   )
@@ -63,9 +69,10 @@
 
 ;; Instructions:
 ;;
-;;  reg is either <reg>, <const> or %VAL0.  If it is <reg>, it can be 'boxed',
-;;  meaning we have one indirection to the shared box.
-;;  register is destination position is always <reg>.
+;;  REG is either <reg>, <const> or %VAL0.
+;;  The register is destination position is always <reg>.
+;;  If a <reg> can be set!, the register is 'boxed'.  The boxing/unboxing
+;;  aren't explicit in the instruction.
 ;;
 ;; General instructions:
 ;;
@@ -73,18 +80,18 @@
 ;; (MOV* r o (dreg ...) sreg) - mv bind (details TBD)
 ;; (LD reg identifier) - global load
 ;; (ST reg identifier) - global store
-;; (CLOSE reg bb)      - make a closure with current env and a basic block BB,
-;;                       leave it in REG.
+;; (CLOSE reg bb)      - make a closure with current env and a basic block
+;;                       BB, leave it in REG.
 ;; (BR reg bb1 bb2)    - if the value of register REG is true, jump to
 ;;                       a basic block BB1.  Otherwise, jump to BB2.
 ;; (JP bb)             - jump to a basic block BB.
 ;; (CONT bb)           - push continuation frame, with BB to be the
 ;;                       'next' basic block.
-;; (CALL bb proc arg ...) - PROC and ARGs are all registers.  Transfer control
-;;                       to PROC. If the continuation of this call is known,
-;;                       BB holds the basic block of the continuation; it is
-;;                       #f if this is a tail call.
-;; (RET reg ...)       - Return, using values in REGs as the results.
+;; (CALL bb proc arg ...) - PROC and ARGs are all registers.
+;;                       Transfer control to PROC. If the continuation of
+;;                       this call is known, BB holds the basic block of
+;;                       the continuation; it is #f if this is a tail call.
+;; (RET reg ...)       - Return, using values in REGs as the result(s).
 ;; (DEF id flags reg)  - insert global binding of ID in the current module
 ;;                       with the value in REG.
 ;;
@@ -171,10 +178,13 @@
 
 ;; Basic blocks:
 ;;   First, we convert IForm to a DG of basic blocks (BBs).
-;;   BB has one entry point and multiple exit point.
+;;   BB has one entry point and multiple exit points.
 
-;; Block environment.  Keep track of allocated registers and blocks
-;; per compile unit (usually each toplevel form)
+;; Block environment.  Keep track of allocated registers and blocks,
+;; per each closure or toplevel form.
+;;
+;; BBs are pushed in BLOCKS slot.  For closures, the first BB, BB#0,
+;; is a dummy BB to track argument registers and has no insns.
 (define-class <benv> ()
   ((name :init-keyword :name)                 ; possible name (for info)
    (registers :init-form '())                 ; (<List> (</> <reg> <const>))
@@ -190,6 +200,10 @@
    (entry :init-value #f)                     ; entry BB
    (blocks :init-value '())                   ; basic blocks
    (clusters :init-value '())                 ; clusters
+   (globals :init-form (make-hash-table default-comparator))
+                                              ; <identifier> -> usage
+                                              ;  usage being a list of
+                                              ;  def, read, write
    (parent :init-keyword :parent)             ; parent benv
    (children :init-value '())))
 
@@ -207,7 +221,10 @@
   ((lvar :init-keyword :lvar)           ; source LVAR; can be #f
    (name :init-keyword :name)           ; given temporary name
    (boxed :init-value #f)               ; #t if this reg should be in a box
-   (blocks :init-value '())             ; BBs using this register.
+   (introduced :init-keyword :introduced) ; BB that introduced this reg.
+   (assigned :init-value '())           ; BBs that assigns this reg. Exclusive
+                                        ;   with introduced.
+   (used :init-value '())               ; BBs using this reg. Includes assigned.
    ))
 
 (define-method write-object ((reg <reg>) port)
@@ -221,19 +238,27 @@
         (and-let1 parent (~ benv'parent)
           (lookup parent lvar))))
   (if-let1 reg (lookup (~ bb'benv) lvar)
-    (touch-reg! bb reg)
-    (let* ([symname (and lvar (unwrap-syntax (lvar-name lvar)))]
-           [name (string->symbol (format "%~d.~d~a" (benv-depth (~ bb'benv))
-                                         (length (~ bb'benv'registers))
+    (use-reg! bb reg)
+    (let* ([benv (~ bb'benv)]
+           [symname (and lvar (unwrap-syntax (lvar-name lvar)))]
+           [name (string->symbol (format "%~d.~d~a" (benv-depth benv)
+                                         (length (~ benv'registers))
                                          (if symname #".~symname" "")))])
-      (rlet1 reg (make <reg> :name name :lvar lvar)
-        (push! (~ bb'benv'registers) reg)
-        (push! (~ reg'blocks) bb)
-        (when lvar (hash-table-put! (~ bb'benv'regmap) lvar reg))))))
+      (rlet1 reg (make <reg> :name name :lvar lvar :introduced bb)
+        (push! (~ benv'registers) reg)
+        (when bb
+          (push! (~ reg'used) bb)
+          (use-reg! bb reg #t))
+        (when lvar (hash-table-put! (~ benv'regmap) lvar reg))))))
 
-(define (touch-reg! bb reg)
-  (when (and (is-a? reg <reg>) (not (memq bb (~ reg'blocks))))
-    (push! (~ reg'blocks) bb))
+;; If reg is used within BB, record the fact.
+(define (use-reg! bb reg :optional (assign? #f))
+  (when (is-a? reg <reg>)
+    (push-unique! (~ reg'used) bb)
+    (when (and assign? (not (memq bb (~ reg'introduced))))
+      (push-unique! (~ reg'assigned) bb))
+    (unless (assq reg (~ bb'reg-use))
+      (push! (~ bb'reg-use) (cons reg (if assign? 'w 'r)))))
   reg)
 
 (define (mark-reg-boxed! reg) (set! (~ reg'boxed) #t))
@@ -262,11 +287,13 @@
    (upstream   :init-keyword :upstream) ; list of upstream blocks
    (downstream :init-value '())         ; list of downstream blocks
    (benv       :init-keyword :benv)
+   (reg-use    :init-value '())         ; register usage alist
+   ;; reg-use keeps how a register is used in the BB first time; it can be
+   ;; either w (written) or r (read).
    (cluster    :init-value #f)
    (entry?     :init-value #f)          ; #t if this BB is entered from outside
    ))
 
-
 (define (make-bb benv . upstreams)
   (rlet1 bb (make <basic-block>
               :benv benv :upstream upstreams :id (length (~ benv'blocks)))
@@ -280,6 +307,9 @@
   (push! (~ from-bb'downstream) to-bb)
   (push! (~ to-bb'upstream) from-bb))
 
+(define (mark-entry-bb! bb)
+  (set! (~ bb'entry?) #t))
+
 (define (bb-name bb)                    ;for debug dump
   (format "BB_~a_~s~a"
           (and (~ bb'cluster) (~ bb'cluster'id))
@@ -291,6 +321,13 @@
 
 (define (last-insn bb) (and (pair? (~ bb'insns)) (car (~ bb'insns))))
 
+;; Returns a vetor of registers that needs to be received from a caller,
+;; if this bb is an entry bb.
+;; Need to be called after register lifetime analysis.
+(define (bb-incoming-regs bb)
+  (list->vector (filter-map (^p (and (eq? (cdr p) 'r) (car p)))
+                            (reverse (~ bb'reg-use)))))
+
 ;; A 'complete' basic block is the one that ends with either
 ;; CALL, BR, JP or RET insn.
 (define (bb-complete? bb)
@@ -305,11 +342,10 @@
 (define-class <cluster> ()
   ((id :init-form (gensym "C")) ;for debugging
    (entry? :init-value #f)      ;#t if this cluster is procedure entry
+   (benv :init-keyword :benv)
    (blocks :init-value '())     ;basic blocks
    (entry-blocks :init-value '()) ; BBs to be jumped from another cluster
-   (upstream :init-value '())   ;upstream clustres
-   (downstream :init-value '()) ;downstream clusters
-   ;; Register classification
+   ;; Register classification.  Used internally.
    ;;  XREGS - Regs outlive this cluster
    ;;  LREGS - Regs local to this cluster
    (xregs :init-form (set eq-comparator))
@@ -317,9 +353,17 @@
    ))
 
 (define (make-cluster benv)
-  (rlet1 c (make <cluster>)
+  (rlet1 c (make <cluster> :benv benv)
     (push! (~ benv'clusters) c)))
 
+;; Does cluster needs dispatch on entry?
+(define (cluster-needs-dispatch? c)
+  (length>? (~ c'entry-blocks) 1))
+
+;; Retunrs a list of cluster registers.  The order is undefined.
+(define (cluster-regs c)
+  (set-fold cons (set-fold cons '() (~ c'lregs)) (~ c'xregs)))
+
 ;;
 ;; Conversion to basic blocks
 ;;
@@ -352,6 +396,7 @@
     (values bb reg)))
 
 (define (pass5b/$DEFINE iform bb benv ctx)
+  (hash-table-push! (~ bb'benv'globals) ($define-id iform) 'def)
   (receive (bb val0) (pass5b/rec ($define-expr iform) bb benv 'normal)
     (push-insn bb `(DEF ,($define-id iform)
                         ,($define-flags iform)
@@ -364,18 +409,21 @@
 (define (pass5b/$LSET iform bb benv ctx)
   (receive (bb val0) (pass5b/rec ($lset-expr iform) bb benv 'normal)
     (let1 r (make-reg bb ($lset-lvar iform))
-      (touch-reg! bb val0)
+      (use-reg! bb val0)
+      (use-reg! bb r #t)
       (push-insn bb `(MOV ,r ,val0))
       (pass5b/return bb ctx r))))
 
 (define (pass5b/$GREF iform bb benv ctx)
   (let1 r (make-reg bb #f)
     (push-insn bb `(LD ,r ,($gref-id iform)))
+    (hash-table-push! (~ bb'benv'globals) ($gref-id iform) 'read)
     (pass5b/return bb ctx r)))
 
 (define (pass5b/$GSET iform bb benv ctx)
   (receive (bb val0) (pass5b/rec ($gset-expr iform) bb benv 'normal)
     (push-insn bb `(ST ,val0 ,($gref-id iform)))
+    (hash-table-push! (~ bb'benv'globals) ($gref-id iform) 'write)
     (pass5b/return bb ctx #f)))
 
 (define (pass5b/$CONST iform bb benv ctx)
@@ -395,13 +443,13 @@
 (define (pass5b/$IF iform bb benv ctx)
   (define (if-branch iform bb benv ctx test-result)
     (if (has-tag? iform $IT)
-      (begin (touch-reg! bb test-result)
+      (begin (use-reg! bb test-result)
              (pass5b/return bb ctx test-result))
       (pass5b/rec iform bb benv ctx)))
   (receive (bb val0) (pass5b/rec ($if-test iform) bb benv 'normal)
     (let ([then-bb (make-bb benv bb)]
           [else-bb (make-bb benv bb)])
-      (touch-reg! bb val0)
+      (use-reg! bb val0)
       (push-insn bb `(BR ,val0 ,then-bb ,else-bb))
       (receive (tbb tval0) (if-branch ($if-then iform) then-bb benv ctx val0)
         (receive (ebb eval0) (if-branch ($if-else iform) else-bb benv ctx val0)
@@ -415,8 +463,8 @@
                  (let* ([cbb (make-bb benv ebb)]
                         [r (and (eq? ctx 'normal) eval0
                                 (rlet1 r (make-reg cbb #f)
-                                  (touch-reg! ebb eval0)
-                                  (touch-reg! ebb r)
+                                  (use-reg! ebb eval0)
+                                  (use-reg! ebb r #t)
                                   (push-insn ebb `(MOV ,r ,eval0))))])
                    (push-insn ebb `(JP ,cbb))
                    (values cbb r)))
@@ -424,8 +472,8 @@
                  (let* ([cbb (make-bb benv tbb)]
                         [r (and (eq? ctx 'normal) tval0
                                 (rlet1 r (make-reg cbb #f)
-                                  (touch-reg! tbb tval0)
-                                  (touch-reg! tbb r)
+                                  (use-reg! tbb tval0)
+                                  (use-reg! tbb r #t)
                                   (push-insn tbb `(MOV ,r ,tval0))))])
                    (push-insn ebb `(JP ,cbb))
                    (values cbb r))
@@ -433,13 +481,13 @@
                         [r (and (eq? ctx 'normal)
                                 (make-reg cbb #f))])
                    (when (and (eq? ctx 'normal) tval0)
-                     (touch-reg! tbb tval0)
-                     (touch-reg! tbb r)
+                     (use-reg! tbb tval0)
+                     (use-reg! tbb r #t)
                      (push-insn tbb `(MOV ,r ,tval0)))
                    (push-insn tbb `(JP ,cbb))
                    (when (and (eq? ctx 'normal) eval0)
-                     (touch-reg! ebb eval0)
-                     (touch-reg! ebb r)
+                     (use-reg! ebb eval0)
+                     (use-reg! ebb r #t)
                      (push-insn ebb `(MOV ,r ,eval0)))
                    (push-insn ebb `(JP ,cbb))
                    (values cbb r))))]))))))
@@ -454,7 +502,8 @@
         (when (memq ($let-type iform) '(rec rec*))
           (mark-reg-boxed! reg))
         (receive (bb val0) (pass5b/rec (car inits) bb benv 'normal)
-          (touch-reg! bb val0)
+          (use-reg! bb val0)
+          (use-reg! bb reg #t)
           (push-insn bb `(MOV ,reg ,val0))
           (loop bb (cdr vars) (cdr inits)))))))
 
@@ -463,19 +512,24 @@
     (receive (bb val0) (pass5b/rec ($receive-expr iform) bb benv 'normal)
       (push-insn bb `(MOV* ,($receive-reqargs iform) ,($receive-optarg iform)
                            ,regs ,val0))
+      (for-each (cut use-reg! bb <> #t) regs)
+      (use-reg! bb val0)
       (pass5b/rec ($receive-body iform) bb benv ctx))))
 
 (define (pass5b/$LAMBDA iform bb benv ctx)
   (let* ([lbenv (make-benv benv ($lambda-name iform))]
          [lbb (~ lbenv'entry)]
-         [reg (make-reg bb #f)])
+         [lbb2 (make-bb lbenv lbb)])
+    (set! (~ lbenv'entry) lbb2) ; skip the dummy block
     (set! (~ lbenv'input-regs)
           (map (cut make-reg lbb <>) ($lambda-lvars iform)))
     (set! (~ lbenv'input-reqargs) ($lambda-reqargs iform))
     (set! (~ lbenv'input-optargs) ($lambda-optarg iform))
-    (receive (cbb val0) (pass5b/rec ($lambda-body iform) lbb lbenv 'tail)
-      (push-insn bb `(CLOSE ,reg ,lbenv))
-      (pass5b/return bb ctx reg))))
+    (push-insn lbb `(JP ,lbb2))
+    (receive (cbb val0) (pass5b/rec ($lambda-body iform) lbb2 lbenv 'tail)
+      (let1 reg (make-reg bb #f)
+        (push-insn bb `(CLOSE ,reg ,lbenv))
+        (pass5b/return bb ctx reg)))))
 
 (define (pass5b/$CLAMBDA iform bb benv ctx)
   (define (generate-closures bb lambdas benv)
@@ -491,7 +545,7 @@
         [((req . _) . rest) (loop rest (if mi (min mi req) req) (max mx req))])))
   (define (create-clambda iform bb cbb ctx)
     (receive (bb clo-regs) (generate-closures bb ($clambda-closures iform) benv)
-      (for-each (cut touch-reg! bb <>) clo-regs)
+      (for-each (cut use-reg! bb <>) clo-regs)
       (receive (minarg maxarg) (reqargs-min-max ($clambda-argcounts iform))
         (let ([closure-list-reg (make-reg bb #f)]
               [minarg-const (make-const bb minarg)]
@@ -551,12 +605,14 @@
         (receive (bb reg) (pass5b/rec (car args) bb benv 'normal)
           (let1 reg (if (eq? reg '%VAL0)
                       (rlet1 reg (make-reg bb #f)
+                        (use-reg! bb reg #t)
                         (push-insn bb `(MOV ,reg %VAL0)))
                       reg)
             (values bb (reverse (cons reg regs)))))
         (receive (bb reg) (pass5b/rec (car args) bb benv 'normal)
           (if (eq? reg '%VAL0)
             (let1 reg (make-reg bb #f)
+              (use-reg! bb reg #t)
               (push-insn bb `(MOV ,reg %VAL0))
               (loop bb (cdr args) (cons reg regs)))
             (loop bb (cdr args) (cons reg regs))))))))
@@ -570,10 +626,11 @@
              [label ($lambda-body proc)]  ; $LABEL mode
              [lbb (make-bb benv bb)])
         (for-each (^[reg lvar]
-                    (touch-reg! bb reg)
+                    (use-reg! bb reg)
                     (let1 lreg (make-reg bb lvar)
                       (unless (eq? lreg reg)
-                        (touch-reg! bb reg)
+                        (use-reg! bb reg)
+                        (use-reg! bb lreg #t)
                         (push-insn bb `(MOV ,lreg ,reg)))))
                   regs ($lambda-lvars proc))
         (push-insn bb `(JP ,lbb))
@@ -586,10 +643,11 @@
              [label ($lambda-body proc)]      ; $LABEL node
              [lbb ($label-label label)])
         (for-each (^[reg lvar]
-                    (touch-reg! bb reg)
+                    (use-reg! bb reg)
                     (let1 lreg (make-reg bb lvar)
                       (unless (eq? lreg reg)
-                        (touch-reg! bb reg)
+                        (use-reg! bb reg)
+                        (use-reg! bb lreg #t)
                         (push-insn bb `(MOV ,lreg ,reg)))))
                   regs ($lambda-lvars proc))
         (link-bb bb lbb)
@@ -599,7 +657,7 @@
     (when cont-bb (set! (~ cont-bb'entry?) #t))
     (receive (bb regs) (pass5b/prepare-args bb benv ($call-args iform))
       (receive (bb proc) (pass5b/rec ($call-proc iform) bb benv 'normal)
-        (for-each (cut touch-reg! bb <>) regs)
+        (for-each (cut use-reg! bb <>) regs)
         (push-insn bb `(CALL ,cont-bb ,proc ,@regs))
         (values (or cont-bb bb) cont-reg))))
   (case ($call-flag iform)
@@ -611,18 +669,20 @@
        [(stmt)   (let1 cbb (make-bb benv bb)
                    (push-insn bb `(CONT ,cbb))
                    (normal-call cbb #f))]
-       [(normal) (let* ([cbb (make-bb benv bb)]
-                        [valreg (make-reg cbb #f)])
+       [(normal) (let* ([cbb (make-bb benv bb)])
                    (push-insn bb `(CONT ,cbb))
-                   (push-insn cbb `(MOV ,valreg %VAL0))
-                   (normal-call cbb valreg))])]))
+                   (receive (cbb _) (normal-call cbb #f)
+                     (let1 valreg (make-reg cbb #f)
+                       (push-insn cbb `(MOV ,valreg %VAL0))
+                       (values cbb valreg))))])]))
 
 ;; NB: Some ASM instructions require c-continuations internally,
 ;; so we should break them up.
 (define (pass5b/$ASM iform bb benv ctx)
   (define (emit bb mnemonic regs)
     (let1 receiver (make-reg bb #f)
-      (for-each (cut touch-reg! bb <>) regs)
+      (for-each (cut use-reg! bb <>) regs)
+      (use-reg! bb receiver #t)
       (push-insn bb `(,mnemonic ,receiver ,@regs))
       (pass5b/return bb ctx receiver)))
   (receive (bb regs) (pass5b/prepare-args bb benv ($asm-args iform) #t)
@@ -673,15 +733,17 @@
 (define (pass5b/builtin-twoargs op iform bb benv ctx)
   (receive (bb regs)
       (pass5b/prepare-args bb benv (list ($*-arg0 iform) ($*-arg1 iform)) #t)
-    (for-each (cut touch-reg! bb <>) regs)
+    (for-each (cut use-reg! bb <>) regs)
     (let1 receiver (make-reg bb #f)
+      (use-reg! bb receiver #t)
       (push-insn bb `(,op ,receiver ,@regs))
       (pass5b/return bb ctx receiver))))
 
 (define (pass5b/$LIST->VECTOR iform bb benv ctx)
   (receive (bb reg0) (pass5b/rec ($*-arg0 iform) bb benv 'normal)
-    (touch-reg! bb reg0)
+    (use-reg! bb reg0)
     (let1 receiver (make-reg bb #f)
+      (use-reg! bb receiver #t)
       (push-insn bb `(LIST->VECTOR ,receiver ,reg0))
       (pass5b/return bb ctx receiver))))
 
@@ -694,11 +756,19 @@
 
 (define (pass5b/builtin-nargs op iform bb benv ctx)
   (receive (bb regs) (pass5b/prepare-args bb benv ($*-args iform) #t)
-    (for-each (cut touch-reg! bb <>) regs)
+    (for-each (cut use-reg! bb <>) regs)
     (let1 receiver (make-reg bb #f)
+      (use-reg! bb receiver #t)
       (push-insn bb `(,op ,receiver ,@regs))
       (pass5b/return bb ctx receiver))))
 
+(define (pass5b/$DYNENV iform bb benv ctx)
+  ;; TODO: A provisional implementation, ignoring denv key&val.
+  ;; A new compiler wraps definition expression with $DYNENV; it only
+  ;; adds debug information, so we can ignore it now.
+  (pass5b/rec ($dynenv-body iform) bb benv ctx)
+  )
+
 (define (pass5b/$IT iform bb benv ctx)
   (error "[Intenral] $IT node should be handled by the parent."))
 
@@ -761,88 +831,145 @@
     (for-each scan (~ benv'children)))
   (scan benv))
 
+;;
+;; lifetime analysis
+;;
+
+;; Make sure a register REG is alive in all paths between its introduction
+;; and its use.
+;; At the beginning, only the BBs that explicitly use REG are marked so.
+;; We have to mark the intermediate BBs of all possible paths.
+;;
+;; We start from a set of BBs that explicitly use REG (input set), and
+;; going upstreams with recording intermediate BBs that are not marked.
+;; Once the path hits a BB that introduces REG, mark all intermediate
+;; BBs.
+
+(define (mark-live-path-bbs! reg)
+  ;; path - hash BB -> (BB1 ...).  Representing we followed BB1 to BB.
+  ;;        If we hit 'using' BB, we mark all BBs chained in this path
+  ;;        as 'using'.  Note that there can be cycles.
+  (define root ($ alist->tree-map
+                  (filter-map (^b (and (not (eq? b (~ reg'introduced)))
+                                       (list b)))
+                              (~ reg'used))
+                  eq-comparator))
+  (define path (make-hash-table eq-comparator))
+  (define (mark-path! bb)
+    (unless (memq bb (~ reg'used))
+      (use-reg! bb reg)
+      (for-each mark-path! (hash-table-get path bb '()))))
+
+  (let repeat ()
+    (and-let* ([p (tree-map-pop-min! root)]
+               [bb0 (car p)])
+      (let loop ([bbs (~ bb0'upstream)])
+        (if (null? bbs)
+          (repeat)
+          (cond [(eq? (car bbs) (~ reg'introduced))
+                 (mark-path! bb0)
+                 (repeat)]
+                [(memq (car bbs) (~ reg'used))
+                 (mark-path! bb0)
+                 (repeat)]
+                [(tree-map-exists? root (car bbs)) (loop (cdr bbs))]
+                [(hash-table-exists? path (car bbs)) (loop (cdr bbs))]
+                [else
+                 (tree-map-put! root (car bbs) '())
+                 (hash-table-push! path (car bbs) bb0)
+                 (loop (cdr bbs))]))))))
+
+(define (mark-all-live-paths! benv)
+  (define (rec benv)
+    (dolist [reg (~ benv'registers)]
+      (when (is-a? reg <reg>)
+        (mark-live-path-bbs! reg)))
+    (for-each rec (~ benv'children)))
+  (rec benv))
+
+;;
+;; BB entry register setup
+;;
+
+(define (bb-assigning-registers bb)
+  (filter (^r (and (is-a? r <reg>)
+                   (memq bb (~ r'assigned))))
+          (~ bb'benv'registers)))
+
+(define (bb-using-registers bb)
+  (filter (^r (and (is-a? r <reg>)
+                   (memq bb (~ r'used))))
+          (~ bb'benv'registers)))
+
+(define (bb-receiving-registers bb)
+  (lset-difference eq? (bb-using-registers bb) (bb-assigning-registers bb)))
+
 ;;
 ;; Basic block clustering
 ;;
 
+
 (define (cluster-bbs! benv)
-  (define (rec benv c bb)
-    (unless (~ bb'cluster)                  ; already visited
-      (push! (~ c'blocks) bb)
-      (set! (~ bb'cluster) c)
-      (match (last-insn bb)
-        [('JP bb1)
-         (rec benv c bb1)
-         (link-clusters! c (~ bb1'cluster))
-         (check-entry-bb! c bb1)]
-        [('BR _ bb1 bb2)
-         (rec benv c bb1)
-         (rec benv c bb2)
-         (link-clusters! c (~ bb1'cluster))
-         (link-clusters! c (~ bb2'cluster))
-         (check-entry-bb! c bb1)
-         (check-entry-bb! c bb2)]
-        [('CALL bb1 proc . regs)
-         (when bb1
-           (if-let1 c2 (~ bb1 'cluster)
-             (link-clusters! c c2)
-             (let1 c2 (make-cluster benv)
-               (push! (~ c2'entry-blocks) bb1)
-               (link-clusters! c c2)
-               (rec benv c2 bb1))))]
-        [_ #f])))
+  (define (visit-1 c bb)
+    (cond
+     [(eq? (~ bb'cluster) c)]
+     [(~ bb'cluster) (error "[internal] something wrong")]
+     [else (set! (~ bb'cluster) c)
+           (push! (~ c'blocks) bb)
+           (visit* c (find-downstream-connections bb))
+           (visit* c (find-upstream-connections bb))]))
+  (define (visit* c bbs)
+    (for-each (cut visit-1 c <>) bbs))
+  (define (find-downstream-connections bb)
+    (match (last-insn bb)
+      [('JP bb1) (list bb1)]
+      [('BR _ bb1 bb2) (list bb1 bb2)]
+      [_ '()]))
+  (define (find-upstream-connections bb)
+    (filter (^[ubb] (memq bb (find-downstream-connections ubb)))
+            (~ bb'upstream)))
+  (define (pick-unvisited bbs)
+    (find (^[bb] (not (~ bb'cluster))) bbs))
+  (define (gen-clusters cs)
+    (if-let1 bb (pick-unvisited (~ benv'blocks))
+      (let1 c (make-cluster benv)
+        (visit-1 c bb)
+        (gen-clusters (cons c cs)))
+      cs))
+  (define (set-entry-blocks! c)
+    (dolist [bb (~ c'blocks)]
+      (when (or (eq? bb (~ benv'entry))
+                (any (^[ubb] (not (memq bb (find-downstream-connections ubb))))
+                     (~ bb'upstream)))
+        (push! (~ c'entry-blocks) bb))))
+
   ;; 1st pass
-  (when (null? (~ benv'clusters))
-    (let1 c (make-cluster benv)
-      (push! (~ c'entry-blocks) (~ benv'entry))
-      (rec benv c (~ benv'entry)))
-    (for-each cluster-bbs! (~ benv'children)))
+  (let1 clusters (gen-clusters '())
+    (for-each set-entry-blocks! clusters)
+    (set! (~ benv'clusters) clusters))
+  (for-each cluster-bbs! (~ benv'children))
   ;; 2nd pass (register classification)
   (dolist [c (~ benv'clusters)]
     (classify-cluster-regs! benv c))
-  (dolist [cbenv (~ benv'children)]
-    (dolist [c (~ cbenv 'clusters)]
-      (classify-cluster-regs! cbenv c)))
-  (adjust-cluster-regs! benv))
-
-(define (link-clusters! upstream downstream)
-  (when (and upstream downstream
-             (not (eq? upstream downstream)))
-    (push-unique! (~ upstream'downstream) downstream)
-    (push-unique! (~ downstream'upstream) upstream)))
+  )
 
 ;; Called when jump from from-cluster to bb.  If bb is not in
 ;; from-cluster, we need to mark bb as an entry bb
 (define (check-entry-bb! from-cluster bb)
   (when (and (~ bb'cluster) (not (eq? from-cluster (~ bb'cluster))))
+    (mark-entry-bb! bb)
     (push-unique! (~ bb'cluster'entry-blocks) bb)))
 
 (define (classify-cluster-regs! benv c)
   (dolist [reg (~ benv'registers)]
     (when (and (is-a? reg <reg>)
-               (any (^b (eq? (~ b'cluster) c)) (~ reg'blocks)))
-      (cond [(and (every (^b (eq? (~ b'cluster) c)) (~ reg'blocks))
+               (any (^b (eq? (~ b'cluster) c)) (~ reg'used)))
+      (cond [(and (every (^b (eq? (~ b'cluster) c)) (~ reg'used))
                   (not (memq reg (~ benv'input-regs))))
              (update! (~ c'lregs) (cut set-adjoin! <> reg))]
             [else
              (update! (~ c'xregs) (cut set-adjoin! <> reg))]))))
 
-;; Xregs used in the downstream clusters have to be carried over
-;; through the cluster.
-(define (adjust-cluster-regs! benv)
-  (define (propagate! c changed)
-    (fold (^[up-c changed]
-            (let1 xset2 (set-union (~ up-c'xregs) (~ c'xregs))
-              (if (set=? xset2 (~ up-c'xregs))
-                changed                 ; no change for this iteration
-                (begin
-                  (set! (~ up-c'xregs) xset2)
-                  #t))))
-          changed (~ c'upstream)))
-  (let loop ()
-    (when (fold propagate! #f (~ benv'clusters))
-      (loop))))
-
 ;;
 ;; For debugging
 ;;
@@ -853,7 +980,7 @@
     #"~(assq-ref benv-alist benv).~(~ benv'name)")
   (define (regname reg)
     (if (is-a? reg <reg>)
-      (if (length>? (~ reg'blocks) 1)
+      (if (length>? (~ reg'used) 1)
         (string->symbol #"~(~ reg'name)*")
         (~ reg'name))
       reg))
@@ -881,17 +1008,19 @@
       [((? (cut memq <> *builtin-ops*) op) . regs)
        (format port "  ~s\n" `(,op ,@(map regname regs)))]
       [_ (format port "??~s\n" insn)]))
-  (define (dump-bb bb)
+  (define (dump-bb bb benv)
     (format port " ~a   [~a> >~a]\n" (bb-name bb)
             (map (cut ~ <> 'id) (reverse (~ bb'upstream)))
             (map (cut ~ <> 'id) (reverse (~ bb'downstream))))
+    (format port "   reg-use: ~s\n"
+            (map (^p (cons (~ (car p)'name) (cdr p))) (~ bb'reg-use)))
     (for-each dump-insn (reverse (~ bb'insns))))
   (define (dump-1 benv)
     (let1 n (x->string (benvname benv))
       (format port "BENV ~a ~a\n"
               n (make-string (- 65 (string-length n)) #\=)))
     (format port "   args: ~s\n" (map regname (~ benv'input-regs)))
-    (for-each dump-bb (reverse (~ benv'blocks)))
+    (for-each (cut dump-bb <> benv) (reverse (~ benv'blocks)))
     (for-each dump-1 (reverse (~ benv'children))))
   (define (assign-benv-names benv)
     (push! benv-alist (cons benv (length benv-alist)))
@@ -900,18 +1029,13 @@
     (format #t "~70,,,'=a\n" "Clusters ")
     (dolist [benv (reverse (map car benv-alist))]
       (dolist [cluster (~ benv'clusters)]
-        (format #t " CLUSTER ~a  [~a> >~a]" (~ cluster'id)
-                (map (cut ~ <> 'id) (~ cluster'upstream))
-                (map (cut ~ <> 'id) (~ cluster'downstream)))
+        (format #t " CLUSTER ~a" (~ cluster'id))
         (if (memq (~ benv'entry) (~ cluster'blocks) )
           (format #t "  ; Entry of ~a\n" (benvname benv))
           (print))
         (format #t "   BBs: ~s\n" (map bb-name (~ cluster'blocks)))
         (format #t "   Entries: ~s\n" (map bb-name (~ cluster'entry-blocks)))
-        (format #t "   xregs: ~s\n" (map (cut ~ <> 'name)
-                                         (set->list (~ cluster'xregs))))
-        (format #t "   lregs: ~s\n" (map (cut ~ <> 'name)
-                                         (set->list (~ cluster'lregs)))))))
+        )))
   (define (dump-registers)
     (format #t "~70,,,'=a\n" "Registers ")
     (dolist [benv (reverse (map car benv-alist))]
@@ -919,7 +1043,7 @@
       (dolist [reg (reverse (~ benv'registers))]
         (when (is-a? reg <reg>)
           (format #t "  ~15,,,,15a ~s\n" (~ reg'name)
-                  (map bb-name (~ reg'blocks)))))))
+                  (map bb-name (~ reg'used)))))))
   (assign-benv-names benv)
   (dump-1 benv)
   (dump-clusters)
@@ -930,10 +1054,10 @@
 (define (compile-b program :optional (mod (vm-current-module)))
   (let* ([cenv (make-bottom-cenv mod)]
          [iform (pass2-4 (pass1 program cenv) mod)])
-    ;(pp-iform iform)
-    (let* ([benv (make-benv #f '%toplevel)]
+    (let* ([benv (make-benv #f (gensym "toplevel"))]
            [bb (pass5b iform benv)])
       (simplify-bbs! benv)
+      (mark-all-live-paths! benv)
       (cluster-bbs! benv)
       benv)))
 
