@@ -1,7 +1,7 @@
 ;;;
 ;;; gauche.cgen.standalone - Create standalone binary
 ;;;
-;;;   Copyright (c) 2014-2022  Shiro Kawai  <shiro@acm.org>
+;;;   Copyright (c) 2014-2023  Shiro Kawai  <shiro@acm.org>
 ;;;
 ;;;   Redistribution and use in source and binary forms, with or without
 ;;;   modification, are permitted provided that the following conditions
@@ -35,8 +35,8 @@
   (use gauche.cgen)
   (use gauche.config)
   (use gauche.process)
-  (use srfi-13)
-  (use srfi-42)
+  (use srfi.13)
+  (use srfi.42)
   (use file.util)
   (export build-standalone)
   )
