@@ -1,7 +1,7 @@
 ;;;
 ;;; gauche.cgen.type - Stub type management
 ;;;
-;;;   Copyright (c) 2004-2022  Shiro Kawai  <shiro@acm.org>
+;;;   Copyright (c) 2004-2023  Shiro Kawai  <shiro@acm.org>
 ;;;
 ;;;   Redistribution and use in source and binary forms, with or without
 ;;;   modification, are permitted provided that the following conditions
@@ -32,7 +32,7 @@
 ;;;
 
 (define-module gauche.cgen.type
-  (use srfi-13)
+  (use srfi.13)
   (use text.tr)
   (use gauche.mop.instance-pool)
   (export <cgen-type> cgen-type-from-name make-cgen-type
@@ -207,10 +207,10 @@
   (%native <size_t>  "Scm_IntegerFitsSizeP" "Scm_IntegerToSize" "Scm_SizeToInteger")
   (%native <ssize_t> "Scm_IntegerFitsSsizeP" "Scm_IntegerToSsize" "Scm_SsizeToInteger")
   (%native <ptrdiff_t> "Scm_IntegerFitsPtrdiffP"  "Scm_IntegerToPtrdiff" "Scm_PtrdiffToInteger")
-
-  ;; TRANSIENT: The following needs to be in after 0.9.12 release
-  ;;(%native <off_t>   "SCM_INTEGERP" "Scm_IntegerToOffset" "Scm_OffsetToInteger")
-  ;;(%native <intptr_t>   "SCM_INTEGERP" "Scm_IntegerToIntptr" "Scm_IntptrToInteger")
+  (%native <off_t>   "Scm_IntegerFitsOffsetP" "Scm_IntegerToOffset" "Scm_OffsetToInteger")
+  ;; TRANSIENT: The following needs to be in after 0.9.13 release
+  ;;(%native <intptr_t>   "Scm_IntegerFitsIntptrP" "Scm_IntegerToIntptr" "Scm_IntptrToInteger")
+  ;;(%native <uintptr_t>   "Scm_IntegerFitsUintptrP" "Scm_IntegerToUintptr" "Scm_UintptrToInteger")
 
   (%native <closure> "SCM_CLOSUREP" "SCM_CLOSURE" "SCM_OBJ")
   (%native <void>    "" "" "SCM_VOID_RETURN_VALUE")
@@ -297,6 +297,7 @@
    (%cclass <compiled-code> "ScmCompiledCode*")
    (%cclass <foreign-pointer> "ScmForeignPointer*")
    (%cclass <box>  "ScmBox*")
+   ;(%cclass <thread-local> "ScmThreadLocal*")
    (%cclass <primitive-parameter> "ScmPrimitiveParameter*")
    (%cclass <dlobj> "ScmDLObj*")
    (%cclass <dlptr> "ScmObj" "Scm_DLPtrP" "SCM_OBJ")
@@ -321,8 +322,7 @@
 ;;
 ;;   cgen-box-tail-expr can be used when the generated value will be
 ;;   immediately returned from SUBR.  The only difference from cgen-box-expr
-;;   is the case for <real>, that can use register-allocated flonumbs
-;;   in that case.
+;;   is the case for flonums, where we can use Scm_VMReturnFlonum.
 ;;
 
 (define (cgen-box-expr type c-expr)
@@ -332,7 +332,7 @@
       #"~|boxer|(~c-expr)")))
 
 (define (cgen-box-tail-expr type c-expr)
-  (let1 boxer (if (memq (~ type'name) '(<real> <float>))
+  (let1 boxer (if (memq (~ type'name) '(<real> <float> <double>))
                 "Scm_VMReturnFlonum"
                 (or (~ type'%boxer) ""))
     (if (cgen-type-maybe? type)
