@@ -1,7 +1,7 @@
 ;;;
 ;;; gauche.interactive - useful stuff in the interactive session
 ;;;
-;;;   Copyright (c) 2000-2022  Shiro Kawai  <shiro@acm.org>
+;;;   Copyright (c) 2000-2023  Shiro Kawai  <shiro@acm.org>
 ;;;
 ;;;   Redistribution and use in source and binary forms, with or without
 ;;;   modification, are permitted provided that the following conditions
@@ -160,7 +160,13 @@
 (define-method describe ((r <rational>))
   (describe-common r)
   (when (ratnum? r)
-    (format #t "  inexact: ~s\n" (inexact r)))
+    (format #t "  inexact: ~s\n" (inexact r))
+    (and-let* ([abs-r (abs r)]
+               [ (> abs-r 1) ]
+               [intpart (floor abs-r)]
+               [mixed `(+ ,intpart ,(- abs-r intpart))]
+               [mixed. (if (negative? r) `(- ,mixed) mixed)])
+      (format #t "    mixed: ~s\n" mixed.)))
   (values))
 
 (define-method describe ((d <real>))
@@ -181,19 +187,31 @@
   (values))
 
 (define-method describe ((g <generic>))
-  (next-method) ; common object description
+  (describe-common g)
+  (describe-slots g)
   (print "methods:")
   (dolist [m (~ g'methods)]
     (let ([spnames (map class-name (~ m'specializers))]
-          [has-optional? (~ m'optional)])
-      (format #t "  ~s\n"
+          [has-optional? (~ m'optional)]
+          [srcloc (source-location m)])
+      (format #t "  ~20s ~a\n"
               (if has-optional?
                 (append spnames '_) ; this works even spnames is ()
-                spnames))))
+                spnames)
+              (if srcloc
+                (format "; ~s:~d" (car srcloc) (cadr srcloc))
+                ""))))
   (and-let1 dis ((with-module gauche.object generic-dispatcher-info) g)
     (format #t "dispatcher:\n  ~s\n" dis))
   (values))
 
+(define-method describe ((m <method>))
+  (describe-common m)
+  (and-let1 source (source-location m)
+    (format #t "Defined at ~s:~d\n" (car source) (cadr source)))
+  (describe-slots m)
+  (values))
+
 (define-method describe ((p <procedure>))
   (describe-common p)
   (and-let1 source (source-location p)
@@ -332,6 +350,11 @@
 
 ;; error printing will be handled by the original read-eval-print-loop
 (define (%evaluator expr env)
+  ;; Kludge - If read edit mode is ON, the final '\n' is consumed by
+  ;; the read-line/edit and the column count isn't reset.  To ensure
+  ;; proper indentation of output during eval, we forcibly reset the
+  ;; column count.
+  (set! (~ (current-output-port)'current-column) 0)
   (guard (e [else (%set-history-exception! e) (raise e)])
     (receive r (eval expr env)
       (%set-history-expr! r)
@@ -339,7 +362,7 @@
 
 ;; <write-controls> used for the printer.
 (define-constant *default-controls*
-  (make-write-controls :length 50 :level 10 :width 79
+  (make-write-controls :length 50 :level 10 :width 79 :string-length 256
                        :pretty (not (sys-getenv "GAUCHE_REPL_NO_PPRINT"))))
 (define *controls* *default-controls*)
 
