@@ -1,7 +1,7 @@
 ;;;
 ;;; interactive/info.scm - online helper
 ;;;
-;;;   Copyright (c) 2000-2021  Shiro Kawai  <shiro@acm.org>
+;;;   Copyright (c) 2000-2023  Shiro Kawai  <shiro@acm.org>
 ;;;
 ;;;   Redistribution and use in source and binary forms, with or without
 ;;;   modification, are permitted provided that the following conditions
@@ -33,7 +33,7 @@
 
 (define-module gauche.interactive.info
   (use scheme.list)
-  (use srfi-13)
+  (use srfi.13)
   (use text.info)
   (use text.pager)
   (use file.util)
@@ -48,54 +48,24 @@
 
 (define *info-file* "gauche-refe.info")
 
-(define-class <repl-info> () ;; a singleton
-  ((info  :init-keyword :info)   ;; <info> or #f
-   (index :init-keyword :index)  ;; hashtable name -> [(node-name line-no)]
-   ))
-
 (define get-info
-  (let1 repl-info
+  (let1 info-doc
       (delay
-        (let ([info  (and-let1 path (find-info-file)
-                       (open-info-file path))]
-              [index (make-hash-table 'string=?)])
-          (if info
-            (begin
-              (dolist [index-page '("Function and Syntax Index"
-                                    "Module Index"
-                                    "Variable Index")]
-                (dolist [p ($ info-parse-menu $ info-get-node info index-page)]
-                  (hash-table-push! index (entry-name (car p)) (cdr p))))
-              ;; class index doesn't have surrounding '<>', but we want to search
-              ;; with them.
-              (dolist [p ($ info-parse-menu $ info-get-node info "Class Index")]
-                (hash-table-push! index #"<~(car p)>" (cdr p)))
-              ($ hash-table-for-each index
-                 ;; reverse v here so that earlier entry listed first
-                 (^[k v]
-                   (hash-table-put! index k (squash-entries (reverse v))))))
+        (if-let1 info-doc (and-let1 path (find-info-file)
+                            (open-info-document path))
+          (begin
+            (info-index-add! info-doc "Function and Syntax Index")
+            (info-index-add! info-doc "Module Index")
+            (info-index-add! info-doc "Variable Index")
+            (info-index-add! info-doc "Class Index" (^e #"<~|e|>"))
+            info-doc)
+          (begin
             (warn "Couldn't find info document in ~s. \
                    Maybe it has't been installed. \
                    Check your Gauche installation.\n"
-                  (get-info-paths)))
-          (make <repl-info>
-            :info info :index index)))
-    (^[] (force repl-info))))
-
-;; We sometimes list API variations using @defunx.  We want to pick only
-;; the first one of such entries.
-(define (squash-entries node&lines)
-  (if (length=? node&lines 1)
-    node&lines
-    (append-map (^[node&lines]
-                  ($ map (^l `(,(caar node&lines) ,(car l)))
-                     $ group-contiguous-sequence $ sort $ map cadr node&lines))
-         (group-collection node&lines :key car :test equal?))))
-
-;; When there are more than one entry with the same name, texinfo appends
-;; " <n>" in the index entry.  This strips that.
-(define (entry-name e)
-  (if-let1 m (#/ <\d+>$/ e) (rxmatch-before m) e))
+                  (get-info-paths))
+            #f)))
+    (^[] (force info-doc))))
 
 (define (get-info-paths)
   (let* ([syspath (cond [(sys-getenv "INFOPATH") => (cut string-split <> #\:)]
@@ -116,7 +86,7 @@
                                     (file-is-readable? #"~|p|.bz2")))))
 
 (define (handle-ambiguous-name entry-name)
-  (let* ([keys (map x->string (hash-table-keys (~ (get-info)'index)))]
+  (let* ([keys (info-index-keys (get-info))]
          [c    (min 2 (- (string-length (x->string entry-name)) 1))]
          [candidates ($ filter-map (^[word dist] (and dist word))
                         keys
@@ -131,8 +101,10 @@
     '()))
 
 (define (get-node&line entry-name)
-  (or (hash-table-get (~ (get-info)'index) (x->string entry-name) #f)
-      (handle-ambiguous-name entry-name)))
+  (let1 es (info-index-ref (get-info) (x->string entry-name))
+    (if (null? es)
+      (handle-ambiguous-name entry-name)
+      es)))
 
 (define (lookup&show key show)
   (match (get-node&line key)
@@ -161,7 +133,7 @@
 (define (info key)
   ($ lookup&show key
      (^[node&line]
-       (let1 node (info-get-node (~ (get-info)'info) (car node&line))
+       (let1 node (info-get-node (get-info) (car node&line))
          (display/pager
           (if (null? (cdr node&line))
             (~ node'content)
@@ -174,15 +146,18 @@
   ($ lookup&show key
      (^[node&line]
        (display/pager
-        (~ (info-get-node (~ (get-info)'info) (car node&line))
-           'content)))))
+        (~ (info-get-node (get-info) (car node&line)) 'content)))))
 
-;;
-;; Search info entries by regexp
-;;
+;; API
+;; Experimental; invoke
+
+
+;;;
+;;; Search info entries by regexp
+;;;
 
 (define (search-entries rx)
-  (sort (filter (^e (rx (car e))) (hash-table->alist (~ (get-info)'index)))
+  (sort (filter (^e (rx (car e))) (info-index->alist (get-info)))
         string<? car))
 
 (define *search-entry-indent* 25)
