@@ -1,7 +1,7 @@
 ;;;
 ;;; interactive/toplevel.scm - toplevel commands
 ;;;
-;;;   Copyright (c) 2015-2022  Shiro Kawai  <shiro@acm.org>
+;;;   Copyright (c) 2015-2023  Shiro Kawai  <shiro@acm.org>
 ;;;
 ;;;   Redistribution and use in source and binary forms, with or without
 ;;;   modification, are permitted provided that the following conditions
@@ -50,7 +50,7 @@
   (use gauche.generator)
   (use gauche.sequence)
   (use gauche.process)
-  (use srfi-13)
+  (use srfi.13)
   (use util.match)
   (use data.trie)
   (export handle-toplevel-command toplevel-command-matches)
@@ -403,7 +403,10 @@
   \n                        the pretty printer.  #f for unlimited.  [default: ~(~ default-pm'width)]\
   \n  base <integer>      - Base radix of showing whole numbers.  [default: ~(~ default-pm'base)]\
   \n  radix <boolean>     - Add radix prefix ('#x' etc.) befors whole numbers.\
-  \n                        [default: ~(~ default-pm'radix)]"
+  \n                        [default: ~(~ default-pm'radix)]\
+  \n  string-length <integer>|#f\
+  \n                      - Max length of string literals before abbreviated."
+
   (^[args]
     (match args
       [() #f]
@@ -414,33 +417,37 @@
        (apply print-mode
               (append-map
                (^[kv]
-                 `(,(if (memq (car kv) '(pretty length level width base radix))
+                 `(,(if (memq (car kv) '(pretty length level width base radix
+                                                string-length))
                       (make-keyword (car kv))
                       (error "print-mode: unrecognized key:" (car kv)))
                    ,(cadr kv)))
                (slices kvs 2)))])
     (let1 c (print-mode)
       (format #t "Current print mode:\n")
-      (format #t "  length : ~3d\n"  (~ c'length))
-      (format #t "   level : ~3d\n"  (~ c'level))
-      (format #t "  pretty : ~3@a\n" (~ c'pretty))
-      (format #t "   width : ~3d\n"  (~ c'width))
-      (format #t "    base : ~3d\n"  (~ c'base))
-      (format #t "   radix : ~3d\n"  (~ c'radix)))
+      (format #t "        length : ~3d\n"  (~ c'length))
+      (format #t "         level : ~3d\n"  (~ c'level))
+      (format #t "        pretty : ~3@a\n" (~ c'pretty))
+      (format #t "         width : ~3d\n"  (~ c'width))
+      (format #t "          base : ~3d\n"  (~ c'base))
+      (format #t "         radix : ~3d\n"  (~ c'radix))
+      (format #t " string-length : ~3d\n"  (~ c'string-length)))
     *no-value*))
 
 (define-toplevel-command (print-all pa) :read
   "\
- \nPrint previous result (*1) without abbreviation.\
+ \nPrint the last result(s) without abbreviation.\
  \nThis is useful when you want the entire S-expression printed out in the\
  \nbuffer, for the copy&paste, etc."
   (^[exprs]
     (match exprs
       ;; TODO: Get console width if possible
       [()
-       (write *1 (write-controls-copy (print-mode)
-                                      :pretty #t :width 79
-                                      :length #f :level #f))
-       (newline)]
+       (dolist [v *1+]
+         (write v (write-controls-copy (print-mode)
+                                       :pretty #t :width 79
+                                       :length #f :level #f
+                                       :string-length #f))
+         (newline))]
       [_ (usage)])
     *no-value*))
