@@ -1,7 +1,7 @@
 ;;;
 ;;; x86_64 minimum assembler
 ;;;
-;;;   Copyright (c) 2015-2022  Shiro Kawai  <shiro@acm.org>
+;;;   Copyright (c) 2015-2023  Shiro Kawai  <shiro@acm.org>
 ;;;
 ;;;   Redistribution and use in source and binary forms, with or without
 ;;;   modification, are permitted provided that the following conditions
@@ -41,8 +41,8 @@
   (use gauche.uvector)
   (use gauche.sequence)
   (use scheme.list)
-  (use srfi-13)
-  (use srfi-42)
+  (use srfi.13)
+  (use srfi.42)
   (use util.match)
   (export asm asm-dump)
   )
