@@ -1,7 +1,7 @@
 ;;; ftp.scm - FTP library
 ;;;
 ;;;  Copyright (c) 2005 OOHASHI Daichi, All rights reserved.
-;;;  Copyright (c) 2007-2022  Shiro Kawai  <shiro@acm.org>
+;;;  Copyright (c) 2007-2023  Shiro Kawai  <shiro@acm.org>
 ;;;
 ;;;  Redistribution and use in source and binary forms, with or without
 ;;;  modification, are permitted provided that the following conditions
@@ -36,8 +36,8 @@
 ;; RFC 2428 FTP Extensions for IPv6 and NATs
 
 (define-module rfc.ftp
-  (use srfi-13)
-  (use srfi-19)
+  (use srfi.13)
+  (use srfi.19)
   (use rfc.uri)
   (use gauche.net)
   (use gauche.logger)
