@@ -1,7 +1,7 @@
 ;;;
 ;;; http.scm - HTTP 1.1
 ;;;
-;;;   Copyright (c) 2000-2022  Shiro Kawai  <shiro@acm.org>
+;;;   Copyright (c) 2000-2023  Shiro Kawai  <shiro@acm.org>
 ;;;
 ;;;   Redistribution and use in source and binary forms, with or without
 ;;;   modification, are permitted provided that the following conditions
@@ -44,8 +44,7 @@
 ;; The features required for typical client usage are implemented first.
 
 (define-module rfc.http
-  (use srfi-11)
-  (use srfi-13)
+  (use srfi.13)
   (use rfc.822)
   (use rfc.uri)
   (use rfc.base64)
@@ -171,7 +170,7 @@
 ;;             appropriate error is thrown from http-request.
 ;;
 ;;   sender  - A procedure that handles sending the request body.
-;;             It takes three arguments: tantative list of headers,
+;;             It takes three arguments: tentative list of headers,
 ;;             the value given to :request-encoding argument,
 ;;             and a procedure HEADER-SINK.
 ;;
