@@ -1,7 +1,7 @@
 ;;;
 ;;; uri.scm - parse and construct URIs
 ;;;
-;;;   Copyright (c) 2000-2022  Shiro Kawai  <shiro@acm.org>
+;;;   Copyright (c) 2000-2023  Shiro Kawai  <shiro@acm.org>
 ;;;
 ;;;   Redistribution and use in source and binary forms, with or without
 ;;;   modification, are permitted provided that the following conditions
@@ -46,7 +46,7 @@
 ;; Also supports 'data' uri scheme specified in RFC2397.
 
 (define-module rfc.uri
-  (use srfi-13)
+  (use srfi.13)
   (use util.match)
   (use gauche.regexp)
   (use gauche.charconv)
