@@ -1,7 +1,7 @@
 ;;;
 ;;; scheme.sort - Sort library (R7RS Red)
 ;;;
-;;;   Copyright (c) 2017-2022  Shiro Kawai  <shiro@acm.org>
+;;;   Copyright (c) 2017-2023  Shiro Kawai  <shiro@acm.org>
 ;;;
 ;;;   Redistribution and use in source and binary forms, with or without
 ;;;   modification, are permitted provided that the following conditions
@@ -31,11 +31,11 @@
 ;;;   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ;;;
 
-;; Originally srfi-132
+;; Originally SRFI-132
 (define-module scheme.sort
   (use gauche.sequence)
   (use gauche.generator)
-  (use srfi-27)   ; we use random selection in vector-select
+  (use srfi.27)   ; we use random selection in vector-select
   (use scheme.vector)
   (export list-sorted? vector-sorted?
           list-sort list-stable-sort
