@@ -2,7 +2,7 @@
 ;;; text.console.windows - windows console control
 ;;;
 ;;;   Copyright (c) 2015  Hamayama  https://github.com/Hamayama
-;;;   Copyright (c) 2016-2022  Shiro Kawai  <shiro@acm.org>
+;;;   Copyright (c) 2016-2023  Shiro Kawai  <shiro@acm.org>
 ;;;
 ;;;   Redistribution and use in source and binary forms, with or without
 ;;;   modification, are permitted provided that the following conditions
@@ -338,7 +338,10 @@
   ;; return the difference of the cursor position y
   (if (and y (<= y 0)) 0 -1))
 
-(define-method query-screen-size ((con <windows-console>))
+(define-method query-screen-size ((con <windows-console>)
+                                  :optional (clear-cache #f))
+  ;; we assume querying screen size is light enough, so we don't cache
+  ;; the result.
   (let1 cinfo (sys-get-console-screen-buffer-info (get-ohandle))
     (values (+ 1 (- (slot-ref cinfo'window.bottom)
                     (slot-ref cinfo'window.top)))
