@@ -1,7 +1,7 @@
 ;;;
 ;;; text.edn - read and write Clojure's Extensible Data Notation
 ;;;
-;;;   Copyright (c) 2018-2022  Shiro Kawai  <shiro@acm.org>
+;;;   Copyright (c) 2018-2023  Shiro Kawai  <shiro@acm.org>
 ;;;
 ;;;   Redistribution and use in source and binary forms, with or without
 ;;;   modification, are permitted provided that the following conditions
@@ -61,7 +61,7 @@
   (use gauche.lazy)
   (use gauche.generator)
   (use gauche.threads)
-  (use srfi-13)
+  (use srfi.13)
   (use scheme.set)
   (use parser.peg)
 
